'use strict';
process.title = 'WWW';
var MAX_RETRY_INTERVAL = 128;
var express = require('express');
var async = require('async');
const { ServiceEnum } = require('@jfrog-pipelines/utility');
const { ConfigFactory } = require('@jfrog-pipelines/configuration');
ConfigFactory.configure(1, '/opt/jfrog/pipelines');
require('./common/logger.js');
var PipelinesAdapter = require('./common/PipelinesAdapter.js');
const { KEEP_ALIVE } = require('./common/constants');
var _ = require('underscore');
var http = require('http');
var util = require('util');
global.request = require('request');
var jfrogAccess = require('@jfrog/access-nodejs-client');
var app = express();
app.disable('x-powered-by');
global.app = app;
global.config = {};
global.config.logLevel = 'info';
global.config.queue = 'www.signals';
global.cache = {
    accessClientContext: {
        token: ''
    }
};
global.request = global.request.defaults({
    headers: {
        'Connection': 'keep-alive',
        'Keep-Alive': `timeout=${KEEP_ALIVE.TIMEOUT_SEC}, max=${KEEP_ALIVE.MAX_CONNECTIONS}`
    }
});
global.redirectSleepInMS = 200;
var who = 'www.app.js';
require('./common/ActErr.js');
require('./common/shipError.js');
logger.info(process.title + ' is checking for dependent environment variables');
init();
process.on('uncaughtException', function (err) {
    shipError(err);
    shutdown();
});
process.on('SIGINT', shutdown);
process.on('SIGTERM', shutdown);
function init() {
    logger.debug(who, 'Starting');
    var initialized = false;
    var retryInterval = 1;
    async.whilst(function () {
        return !initialized;
    }, function (done) {
        var bag = {
            who: who,
            accessClient: null,
            accessClientContext: {}
        };
        async.series([
            initializeConfig.bind(null, bag),
            setSystemConfig.bind(null, bag),
            initializeSUAdapter.bind(null, bag),
            initializeAccessClient.bind(null, bag),
            registerToAccessRouter.bind(null, bag),
            setLogLevel.bind(null, bag),
        ], function (err) {
            if (err) {
                shipError(err);
                retryInterval = retryInterval * 2;
                if (retryInterval > MAX_RETRY_INTERVAL)
                    retryInterval = 2;
                logger.info(who, 'Retrying in', retryInterval, 'seconds');
                return setTimeout(done, retryInterval * 1000);
            }
            initialized = true;
            return done();
        });
    }, function (err) {
        startApp();
    });
}
function initializeConfig(bag, next) {
    var who = bag.who + '|' + initializeConfig.name;
    logger.debug(who, 'Inside');
    var configErrors = [];
    var ConfigHandler = require('./common/ConfigHandler.js');
    var globalConfig = new ConfigHandler().globalConfig;
    if (!globalConfig.apiToken)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'apiToken is not defined'));
    if (!globalConfig.pipelinesNodeId)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'pipelinesNodeId is not defined'));
    if (!globalConfig.apiUrl)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'apiUrl is not defined'));
    if (!globalConfig.publicApiUrl)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'publicApiUrl is not defined'));
    if (!globalConfig.rootAmqpUrl)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'amqpUrl is not defined'));
    if (!globalConfig.amqpExchange)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'amqpExchange is not defined'));
    if (!globalConfig.redisUrl)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'redisUrl is not defined'));
    if (!globalConfig.sessionSecret)
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'sessionSecret is not defined'));
    if (!_.has(globalConfig, 'isSaas'))
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'isSaas is not defined'));
    if (!_.has(globalConfig, 'redisClusterEnabled'))
        configErrors.push(new ActErr(who, ActErr.ParamNotFound, 'redisClusterEnabled is not defined'));
    global.config = _.extend(global.config, globalConfig);
    global.pipelinesNodeId = global.config.pipelinesNodeId;
    global.config.amqpUrl = global.config.rootAmqpUrl;
    if (configErrors.length > 0) {
        configErrors.forEach(function (err) {
            shipError(err);
        });
        return next(true);
    }
    return next();
}
function setSystemConfig(bag, next) {
    var who = bag.who + '|' + setSystemConfig.name;
    logger.debug(who, 'Inside');
    global.config.systemConfig = {
        defaultMinionCount: global.config.defaultMinionCount,
        stepTimeoutMS: global.config.stepTimeoutMS,
        allowDynamicNodes: global.config.allowDynamicNodes,
        releaseVersion: global.config.controlplaneVersion,
        controlplaneVersion: global.config.controlplaneVersion,
        buildplaneVersion: global.config.buildplaneVersion,
        allowCustomNodes: global.config.allowCustomNodes,
        allowSystemNodes: global.config.allowSystemNodes,
        maxDiskUsagePercentage: global.config.maxDiskUsagePercentage,
        nodeCacheIntervalMS: global.config.nodeCacheIntervalMS,
        nodeStopDayOfWeek: global.config.nodeStopDayOfWeek,
        nodeStopIntervalDays: global.config.nodeStopIntervalDays,
        supportEmailAddress: global.config.supportEmailAddress,
        rtServiceId: global.config.rtServiceId,
    };
    logger.info('Dependent variable check succeeded for ' +
        process.title);
    return next();
}
function initializeSUAdapter(bag, next) {
    var who = bag.who + '|' + initializeSUAdapter.name;
    logger.debug(who, 'Inside');
    bag.suApiAdapter = new PipelinesAdapter(global.config.apiToken);
    return next();
}
function initializeAccessClient(bag, next) {
    var who = bag.who + '|' + initializeAccessClient.name;
    logger.debug(who, 'Inside');
    var options = {
        joinKey: global.config.rtRouterJoinKey,
        nodeId: global.config.nodeId,
        serviceId: global.config.wwwServiceId,
        routerBaseUrl: '127.0.0.1:' + global.config.rtRouterAccessPort,
        accessBaseUrl: 'http://127.0.0.1:' + global.config.rtRouterAccessPort +
            '/access',
        accessGrpcBaseUrl: '127.0.0.1:' + global.config.rtRouterAccessPort,
        numberOfRetries: 5,
        logger: logger,
        restHeaders: { 'Connection': 'keep-alive', 'Keep-Alive': `timeout=${KEEP_ALIVE.TIMEOUT_SEC}, max=${KEEP_ALIVE.MAX_CONNECTIONS}` },
        tokenVerifyCacheTTLInSec: 300
    };
    var revision = global.config.controlplaneVersion.replace(/[.]/g, 0);
    revision = revision.replace(/[-m]/g, '');
    global.cache.accessClientContext.serviceVersion =
        global.config.controlplaneVersion;
    global.cache.accessClientContext.serviceRevision = revision;
    global.client = {};
    global.client.access = new jfrogAccess.AccessClient(options);
    global.client.access.join(global.cache.accessClientContext).then(function (res) {
        logger.trace(who, 'Successfully initialized Access client');
        global.cache.accessClientContext.token = res;
        return next();
    }).catch(function (err) {
        logger.warn(bag.who, 'Failed to initialize Access client', err);
        return next(new ActErr(who, ActErr.InternalServer, 'Failed to initialize Access client with error:', err));
    });
}
function registerToAccessRouter(bag, next) {
    var who = bag.who + '|' + registerToAccessRouter.name;
    logger.debug(who, 'Inside');
    var registerRequest = new jfrogAccess.RegisterServiceRequest();
    registerRequest.setNodeId(global.config.nodeId);
    registerRequest.setServiceId(global.config.wwwServiceId);
    var routePath = new jfrogAccess.RoutePath();
    routePath.setMatch('pipelines/www/(.*)');
    routePath.setReplace('/$1');
    var routePathWithoutSlash = new jfrogAccess.RoutePath();
    routePathWithoutSlash.setMatch('pipelines/www');
    routePathWithoutSlash.setReplace('/');
    var wwwEndPoint = new jfrogAccess.LocalEndpoint();
    wwwEndPoint.setPort(global.config.wwwPort);
    wwwEndPoint.setProtocol(jfrogAccess.EndpointProtocol.HTTP1);
    wwwEndPoint.setPathsList([routePath, routePathWithoutSlash]);
    var healthCheckEndpoint = new jfrogAccess.Endpoint();
    healthCheckEndpoint.setPort(global.config.wwwPort);
    healthCheckEndpoint.setPath('/');
    registerRequest.setEndpointsList([wwwEndPoint]);
    registerRequest.setHealthCheckEndpoint(healthCheckEndpoint);
    global.client.access.register(global.cache.accessClientContext, registerRequest).then(function () {
        logger.debug(who, 'Successfully Registered to access router');
        return next();
    }).catch(function (err) {
        return next(new ActErr(who, ActErr.InternalServer, 'Failed to register to access router with error: ', err));
    });
}
function setLogLevel(bag, next) {
    var who = bag.who + '|' + setLogLevel.name;
    logger.trace(who, 'Inside');
    logger.info('----------------------------------------------------------------------------');
    logger.info('----------------------------------------------------------------------------');
    logger.info('--------------------- WWW Successfully booted ------------------------------');
    logger.info('----------------------------------------------------------------------------');
    logger.info('----------------------------------------------------------------------------');
    logger.trace('Setting log level as ' + global.config.runMode);
    if (global.config.runMode === 'devmode')
        global.config.logLevel = 'trace';
    else if (global.config.runMode === 'beta')
        global.config.logLevel = 'debug';
    else if (global.config.runMode === 'production' ||
        global.config.runMode === 'onebox')
        global.config.logLevel = 'warn';
    else {
        logger.error(util.format('Invalid runMode: %s.  Defaulting to production mode.', runMode));
        global.config.logLevel = 'warn';
    }
    require('./common/logger').changeOldLoggerLevel(global.config.logLevel);
    return next();
}
function shutdown(signal) {
    if (signal)
        process.removeListener(signal, shutdown);
    async.series([
        stopListening,
        disconnectAccessRouter
    ], function (err) {
        if (err) {
            shipError(err);
        }
        if (signal)
            process.kill(process.pid, signal);
        else
            process.exit(1);
    });
}
function stopListening(next) {
    if (!global.httpServer)
        return next();
    var who = 'www.app.js|' + stopListening.name;
    logger.debug(who, 'Inside');
    global.httpServer.close(function (err) {
        if (err)
            logger.warn(who, 'Failed to close connections with error:', err);
        else
            logger.debug(who, 'Successfully closed connections');
        return next();
    });
}
function disconnectAccessRouter(next) {
    if (!global.client || !global.client.access)
        return next();
    var who = 'www.app.js|' + disconnectAccessRouter.name;
    logger.debug(who, 'Inside');
    var unregisterRequest = new jfrogAccess.UnregisterServiceRequest();
    unregisterRequest.setNodeId(global.config.nodeId);
    unregisterRequest.setServiceId(global.config.wwwServiceId);
    global.client.access.unregister(global.cache.accessClientContext, unregisterRequest).then(function () {
        logger.debug(who, 'Successfully unregistered with access router');
        return next();
    }).catch(function (err) {
        logger.warn(who, 'Failed to unregister with access router with error:', err);
        return next();
    });
}
function startApp() {
    var started = false;
    var retryInterval = 1;
    function waitAndRetry(callback) {
        retryInterval = retryInterval * 2;
        if (retryInterval > MAX_RETRY_INTERVAL)
            retryInterval = 2;
        logger.info(who, 'Retrying in', retryInterval, 'seconds');
        setTimeout(callback, retryInterval * 1000);
    }
    async.whilst(function () {
        return !started;
    }, function (done) {
        try {
            global.httpServer = http.Server(app);
        }
        catch (e) {
            shipError(e);
            return waitAndRetry(done);
        }
        var MicroService = require('./system/MicroService.js');
        global.MicroService = new MicroService(function (err) {
            if (err) {
                shipError(err);
                return waitAndRetry(done);
            }
            try {
                global.httpServer.listen(global.config.wwwPort);
            }
            catch (e) {
                shipError(e);
                return waitAndRetry(done);
            }
            global.httpServer.on('listening', function () {
                logger.info('HTTP Server is running on port ' + global.config.wwwPort);
            });
            global.httpServer.on('error', function (err) {
                shipError(err);
                shutdown();
            });
        });
    }, function (err) {
        shipError(err);
    });
}
//# sourceMappingURL=www.app.js.map